@@ -18,6 +18,15 @@ To do:
 - generate input (including sub-folder mesh and field) and output folders
 - generate mesh file (DEM.txt) and field files
 - divide model domain into small sections if multiple GPU is used
+Structure:
+   class InputHipims
+       - Initialize an object: __init__
+       - set model parameters: 
+           set_boundary_condition, set_parameter, set_rainfall, 
+           set_gauges_position, set_case_folder, set_runtime, set_device_no,
+           add_user_defined_parameter, set_num_of_sections
+
+
 """
 __author__ = "Xiaodong Ming"
 import os
@@ -33,7 +42,7 @@ from .ModelSummary import ModelSummary
 from .spatial_analysis import sub2map
 from . import indep_functions as indep_f
 from . import rainfall_processing as rp
-#%% grid data for HiPIMS input format
+#%% definition of class InputHipims
 class InputHipims:
     """To define input files for a HiPIMS flood model case
     Read data, process data, write input files, and save data of a model case.
@@ -61,7 +70,6 @@ class InputHipims:
         _outline_cell_subs: (tuple, int numpy) two numpy array indicating rows
             and cols of outline cells on the DEM array and sorted based on the
             valid_cell_id from small to large
-        _global_header: (dict) header of the DEM for the whole model domain
     Methods (public):
         set_parameter: set grid-based parameters
         set_boundary_condition: set boundary condition with a boundary list
@@ -90,6 +98,8 @@ class InputHipims:
         Boundary: provide information of boundary conditions
         ModelSummary: record basic information of an object of InputHiPIMS
     """
+#%%****************************************************************************
+#***************************initialize an object*******************************
     # default parameters
     attributes_default = {'h0':0, 'hU0x':0, 'hU0y':0,
                           'precipitation':0,
@@ -127,11 +137,9 @@ class InputHipims:
         # divide model domain to several sections if it is not a sub section
         # each section contains a "HiPIMS_IO_class.InputHipimsSub" object
         if isinstance(self, InputHipimsSub):
-#            print('child class object '+str(self.section_id))
             pass
         else:
             self.__divide_grid()
-            self._global_header = self.DEM.header        
         self.set_case_folder() # set data_folders
         self.set_device_no() # set the device number
         self.set_boundary_condition(outline_boundary='fall')
@@ -145,7 +153,7 @@ class InputHipims:
         time_str = self.birthday.strftime('%Y-%m-%d %H:%M:%S')
         return  self.__class__.__name__+' object created on '+ time_str
 #    __repr__ = __str__
-#******************************************************************************
+#%%****************************************************************************
 #************************Setup the object**************************************
     def set_boundary_condition(self, boundary_list=None,
                                outline_boundary='fall'):
@@ -180,7 +188,7 @@ class InputHipims:
         valid_subs = self._valid_cell_subs
         outline_subs = self._outline_cell_subs
         if not isinstance(self, InputHipimsSub):
-            dem_header = self._global_header
+            dem_header = self.header
         # add the subsripts and id of boundary cells on the domain grid
             bound_obj._fetch_boundary_cells(valid_subs,
                                             outline_subs, dem_header)
@@ -271,7 +279,7 @@ class InputHipims:
             pos_X = gauges_pos[:,0]
             pos_Y = gauges_pos[:,1]
             for obj_section in self.Sections:
-                extent = obj_section.Raster.extent
+                extent = obj_section.DEM.extent
                 ind_x = np.logical_and(pos_X >= extent[0], pos_X <= extent[1])
                 ind_y = np.logical_and(pos_Y >= extent[2], pos_Y <= extent[3])
                 ind = np.where(np.logical_and(ind_x, ind_y))
@@ -285,20 +293,16 @@ class InputHipims:
         make_dir: True|False create folders if it is True
         """
         # to change case_folder
-        if new_folder is not None:
-            if new_folder.endswith('/'):
-                new_folder = new_folder[:-1]
-            if new_folder == 'cwd':
-                new_folder = os.getcwd()
-            self.case_folder = new_folder
-            # for multiple GPUs
-            if hasattr(self, 'Sections'):
-                for obj in self.Sections:
-                    sub_case_folder = new_folder+'/'+str(obj.section_id)
-                    obj.set_case_folder(sub_case_folder) 
-        # for single gpu
+        if new_folder is None:
+            new_folder = self.case_folder
+        self.case_folder = new_folder
         self.data_folders = _create_io_folders(self.case_folder,
                                                make_dir)
+        # for multiple GPUs
+        if hasattr(self, 'Sections'):
+            for obj in self.Sections:
+                sub_case_folder = new_folder+'/'+str(obj.section_id)
+                obj.set_case_folder(sub_case_folder)                        
         if hasattr(self, 'Summary'):
             self.Summary.set_param('Case folder', self.case_folder)
     
@@ -337,33 +341,57 @@ class InputHipims:
         self.attributes[param_name] = param_value
         print(param_name+ 'is added to the InputHipims object')
         self.Summary.add_param_infor(param_name, param_value)
+    
+    def set_num_of_sections(self, num_of_sections):
+        """ set the number of divided sections to run a case
+        can transfer single-gpu to multi-gpu and the opposite way
+        """
+#        obj_new = copy.deepcopy(self)
+        self.num_of_sections = num_of_sections
+        self.set_device_no()
+        if num_of_sections==1: # to single GPU
+            if hasattr(self, 'Sections'):
+                del self.Sections
+        else: # to multiple GPU
+            self.__divide_grid()
+            outline_boundary = self.Boundary.outline_boundary
+            self.set_boundary_condition(outline_boundary=outline_boundary)
+            self.set_gauges_position()
+            self.Boundary._divide_domain(self)
+        self.set_case_folder(self.case_folder)
+        self.birthday = datetime.now()
+        self.Summary.set_param('Number of Sections', str(num_of_sections))
+        time_str = self.birthday.strftime('%Y-%m-%d %H:%M:%S')
+        self.Summary.set_param('Birthday', time_str)
+        
 
     def decomposite_domain(self, num_of_sections):
+        """ divide a single-gpu case into a multi-gpu case
+        """
         obj_mg = copy.deepcopy(self)
-        if isinstance(self, InputHipims):
-            obj_mg.num_of_sections = num_of_sections
-            obj_mg.set_device_no()
-            obj_mg._global_header = obj_mg.Raster.header
-            obj_mg.__divide_grid()
-            obj_mg.set_case_folder() # set data_folders
-            outline_boundary = obj_mg.Boundary.outline_boundary
-            obj_mg.set_boundary_condition(outline_boundary=outline_boundary)
-            obj_mg.set_gauges_position()
-            obj_mg.Boundary._divide_domain(obj_mg)
-            obj_mg.birthday = datetime.now()
-        else:
-            raise ValueError('The object cannot be decomposited!')
+        obj_mg.num_of_sections = num_of_sections
+        obj_mg.set_device_no()
+        obj_mg.__divide_grid()
+        obj_mg.set_case_folder() # set data_folders
+        outline_boundary = obj_mg.Boundary.outline_boundary
+        obj_mg.set_boundary_condition(outline_boundary=outline_boundary)
+        obj_mg.set_gauges_position()
+        obj_mg.Boundary._divide_domain(obj_mg)
+        obj_mg.birthday = datetime.now()
         return obj_mg
 
+#%%****************************************************************************
+#************************Write input files*************************************
     def write_input_files(self, file_tag=None):
         """ Write input files
         To classify the input files and call functions needed to write each
             input files
-        file_tag: 'all'|'z', 'h', 'hU', 'manning', 'sewer_sink',
+        file_tag: str or list of str including
+                  ['z', 'h', 'hU', 'manning', 'sewer_sink',
                         'cumulative_depth', 'hydraulic_conductivity',
                         'capillary_head', 'water_content_diff'
                         'precipitation_mask', 'precipitation_source',
-                        'boundary_condition', 'gauges_pos'
+                        'boundary_condition', 'gauges_pos']
         """
         self._make_data_dirs()
         grid_files = InputHipims.grid_files
@@ -496,7 +524,7 @@ class InputHipims:
         else:
             for obj_section in self.Sections:
                 file_name = obj_section.data_folders['mesh']+'DEM.txt'
-                obj_section.Raster.write_asc(file_name)
+                obj_section.DEM.write_asc(file_name)
         self.Summary.write_readme(self.case_folder+'/readme.txt')
         print('DEM.txt created')
     
@@ -507,6 +535,7 @@ class InputHipims:
             time_values = self.times
         indep_f.write_times_setup(self.case_folder, self.num_of_sections,
                                   time_values)
+        self.Summary.write_readme(self.case_folder+'/readme.txt')
     
     def write_device_file(self, device_no=None):
         """Create device_setup.dat for choosing GPU number to run the model
@@ -520,13 +549,18 @@ class InputHipims:
         """ Save object as a pickle file
         """
         indep_f.save_object(self, file_name, compression=True)
-#------------------------------------------------------------------------------
+
+#%%****************************************************************************
 #******************************* Visualization ********************************
-#------------------------------------------------------------------------------
     def domain_show(self, figname=None, dpi=None, **kwargs):
         """Show domain map of the object
         """
-        fig, ax = self.DEM.mapshow(**kwargs)
+        obj_dem = copy.deepcopy(self.DEM)
+        if hasattr(self, 'Sections'):
+            for obj_sub in self.Sections:
+                overlayed_subs = obj_sub.overlayed_cell_subs_global
+                obj_dem.array[overlayed_subs] = np.nan            
+        fig, ax = obj_dem.mapshow(**kwargs)
         cell_subs = self.Boundary.cell_subs
         legends = []
         num = 0
@@ -573,9 +607,9 @@ class InputHipims:
                                     method=method)
         rp.plot_time_series(plot_data, method=method)
         return plot_data
-#------------------------------------------------------------------------------
+
+#%%****************************************************************************
 #*************************** Protected methods ********************************
-#------------------------------------------------------------------------------
     def _get_cell_subs(self, dem_array=None):
         """ To get valid_cell_subs and outline_cell_subs for the object
         To get the subscripts of each valid cell on grid
@@ -610,7 +644,6 @@ class InputHipims:
                 return 1 # do not divide if num_of_sections is 1
         num_of_sections = self.num_of_sections
         dem_header = self.header
-        self._global_header = dem_header
         # subscripts of the split row [0, 1,...] from bottom to top
         split_rows = _get_split_rows(self.DEM.array, num_of_sections)
         array_local, header_local = \
@@ -618,7 +651,7 @@ class InputHipims:
         # to receive InputHipimsSub objects for sections
         Sections = []
         section_sequence = np.arange(num_of_sections)
-        header_global = self._global_header
+        header_global = dem_header
         for i in section_sequence:  # from bottom to top
             case_folder = self.case_folder+'/'+str(i)
             # create a sub object of InputHipims
@@ -648,6 +681,12 @@ class InputHipims:
                                 'bottom_high':bottom_h[0],
                                 'bottom_low':bottom_l[0]}
             sub_hipims.overlayed_id = overlayed_id
+            all_ids = list(overlayed_id.values())
+            all_ids = np.concatenate(all_ids).ravel()
+            all_ids.sort()
+            overlayed_cell_subs_global = (valid_subs_global[0][all_ids],
+                                          valid_subs_global[1][all_ids])
+            sub_hipims.overlayed_cell_subs_global = overlayed_cell_subs_global
             Sections.append(sub_hipims)
         # reset global var section_id of InputHipimsSub
         InputHipimsSub.section_id = 0
@@ -905,7 +944,8 @@ class InputHipims:
             for file in file_names:
                 shutil.copy2(file, field_dir)
 
-#%% sub-class definition
+#%%****************************************************************************
+#************************sub-class definition**********************************
 class InputHipimsSub(InputHipims):
     """object for each section, child class of InputHipims
     Attributes:
@@ -926,8 +966,8 @@ class InputHipimsSub(InputHipims):
         dem_data = Raster(array=dem_array, header=header)
         super().__init__(dem_data, num_of_sections, case_folder)
 
-#%% ===================================Static method===========================
-
+#%%****************************************************************************
+#********************************Static method*********************************
 def _cell_subs_convertor(input_cell_subs, header_global,
                          header_local, to_global=True):
     """
@@ -1091,7 +1131,7 @@ def _split_array_by_rows(input_array, header, split_rows, overlayed_rows=2):
         sub_header['yllcorner'] = sub_yllcorner
         header_local.append(sub_header)
     return array_local, header_local
-#%create IO Folders for each case
+
 def _create_io_folders(case_folder, make_dir=False):
     """ create Input-Output path for a Hipims case
         (compatible for single/multi-GPU)
